# -*- coding: UTF-8 -*-
from icv.utils import IS_PY3
try:
    if IS_PY3:
        from http.server import BaseHTTPRequestHandler, SimpleHTTPRequestHandler
    else:
        from SimpleHTTPServer import BaseHTTPRequestHandler, SimpleHTTPRequestHandler
except ImportError:
    raise ModuleNotFoundError

import io
from urllib.parse import urlparse
from ..exceptions import HttpParseException
import os
import re
import json
from ..status_code import HTTP_STATUS_CODES
from http import HTTPStatus
import socket
from .context import Context

def api(ctx):
    data = ctx.request.data_json
    print("======> data_json:",data)
    print("======> data_post:",ctx.request.data_post)
    print("======> data:",ctx.request.data)
    if data and "name" in data:
        print("yes!!!!!")
    ctx.response.send("post api")

def error(ctx):
    ctx.response.send({"error":"you are wrong"},500)

def ok_ke(ctx):
    ctx.response.send("post ok ke")

def hello(ctx):
    ctx.response.send("get hello")

def _init_routes():  # 初始化
    global routes
    global method_routes

    routes = {
        "/api": ("POST", api),
        "/error": (["POST", "PUT"], error),
        "/ok/ke": ("POST", ok_ke),
        "/hello": ("GET", hello),
    }
    method_routes = {
        "POST": [
            ("/api", api),
            ("/ok/ke", ok_ke)
        ],
        "GET": [
            ("/hello", hello)
        ]
    }

def add_route(action,methods,function):
    routes[action] = (methods, function)
    if isinstance(methods, str):
        method_routes[methods].append((action, function))
    for m in list(methods):
        if m not in method_routes:
            method_routes[m] = []
        method_routes[m].append((action, function))

class IcvHandler(SimpleHTTPRequestHandler):
    def _prepare_response(self,status_code=200,content_type="text/plain",**kwargs):
        self.send_response(status_code)
        self.send_header('Content-Type', content_type)
        for k in kwargs:
            self.send_header(k,kwargs[k])
        self.end_headers()

    def render(self,html,enc="utf-8"):
        print("#### render html:",html)
        if os.path.isfile(html):
            encoded = open(html,"r").read().encode(enc, 'surrogateescape')
        else:
            # encoded = html.encode(enc, 'surrogateescape')
            return self.error_404()
        f = io.BytesIO()
        f.write(encoded)
        f.seek(0)
        self.send_response(HTTPStatus.OK)
        self.send_header("Content-type", "text/html; charset=%s" % enc)
        self.send_header("Content-Length", str(len(encoded)))
        self.end_headers()
        self.wfile.write(encoded)

    def error_404(self):
        self.error(404,"404 Not Found.")

    def error(self,status_code=500,errMsg=None):
        errMsg = errMsg if errMsg and isinstance(errMsg,str) else "Internal Server Error"
        self.response(errMsg,status_code=status_code)

    def response(self,data,status_code=200,**kwargs):
        if isinstance(data,dict):
            res = json.dumps(data).encode("utf-8")
        elif isinstance(data,str):
            res = data.encode("utf-8")
        elif isinstance(data,bytes):
            res = data
        else:
            return self.error()

        content_type = None
        if isinstance(data,str):
            content_type = "text/plain;charset=utf-8"
        elif isinstance(data,dict):
            content_type = "application/json;charset=utf-8"
        else:
            if "Content-Type" in kwargs:
                content_type = kwargs["Content-Type"]
            elif "content-type" in kwargs:
                content_type = kwargs["content-type"]

        if content_type is not None:
            self._prepare_response(status_code=status_code,content_type=content_type,**kwargs)
        else:
            self._prepare_response(status_code=status_code,**kwargs)
        self.wfile.write(res)

    def _build_ctx(self):
        try:
            ctx = Context(self, self.command, dict(self.headers._headers))
            ctx.request.raw_path = self.path
            content_length = int(self.headers["Content-Length"]) if "Content-Length" in self.headers else 0
            post_data = self.rfile.read(content_length)
            ctx.request.data = post_data
            return ctx
        except HttpParseException as e:
            self.error(e.code,e.msg)
            self.finish()
            print("===> HttpParseException:",e.msg)
        except Exception as e:
            self.error()
            print("===> Exception:",e)

    def handle_one_request(self):
        try:
            self.raw_requestline = self.rfile.readline(65537)
            if len(self.raw_requestline) > 65536:
                self.requestline = ''
                self.request_version = ''
                self.command = ''
                self.send_error(HTTPStatus.REQUEST_URI_TOO_LONG)
                return
            if not self.raw_requestline:
                self.close_connection = True
                return
            if not self.parse_request():
                # An error code has been sent, just exit
                return
            mname = 'do_' + self.command
            if not hasattr(self, mname):
                self.send_error(
                    HTTPStatus.NOT_IMPLEMENTED,
                    "Unsupported method (%r)" % self.command)
                return

            up = urlparse(self.path)
            target = self._match(up.path.rstrip("/"))
            if target is None:
                if up.path.rstrip("/") == "":
                    self.response({
                        "up":"true",
                        "method":self.command,
                        "path":up.path,
                        "params":up.params,
                        "query":up.query,
                    })
                    return
                self.send_error(
                    HTTPStatus.NOT_FOUND,
                    "Not Found handler (%r)" % self.path
                )
                return

            ms,func,_ = target
            if isinstance(ms,str) and self.command != ms:
                self.send_error(
                    HTTPStatus.METHOD_NOT_ALLOWED,
                    "method not allowed (%r)" % self.command
                )
                return

            if isinstance(ms,(list,tuple)) and self.command not in ms:
                self.send_error(
                    HTTPStatus.METHOD_NOT_ALLOWED,
                    "method not allowed (%r)" % self.command
                )

            method = getattr(self, mname)
            method(func)
            self.wfile.flush() #actually send the response if not already done.
        except socket.timeout as e:
            #a read or a write timed out.  Discard this connection
            self.log_error("Request timed out: %r", e)
            self.close_connection = True
            return

    def _match(self, rpath):
        print("now math path:",rpath)
        print("routes:",routes)
        for action in routes:
            handler_method,handler_func = routes[action]
            result = re.compile("^" + str(action) + "$").match(str(rpath))
            if result:
                return handler_method,handler_func,[x for x in result.groups()]
        # No match, return None
        print("=====> not match path:",rpath)
        return None

    def _filter(self,method=None):
        target = self._match(self.path)
        if target is None:
            self.error(404,HTTP_STATUS_CODES[404][0])
            self.finish()
            return None

        if method is None:
            return target

        handler_method,_,_ = target
        if isinstance(handler_method,str):
            if handler_method.upper() != str(method).upper():
                self.error(501, HTTP_STATUS_CODES[501][0])
                self.finish()
                return None
        else:
            if str(method).upper() not in handler_method:
                self.error(501, HTTP_STATUS_CODES[501][0])
                self.finish()
                return None

        return target

    def do_GET(self,handler=None):
        if handler is None:
            self.error(400,"handler not exist!")
            return

        handler(self._build_ctx())

    def do_PUT(self,handler=None):
        if handler is None:
            self.error(400, "handler not exist!")
            return

        handler(self._build_ctx())

    def do_DELETE(self,handler=None):
        if handler is None:
            self.error(400, "handler not exist!")
            return

        handler(self._build_ctx())

    def do_POST(self,handler=None):
        if handler is None:
            self.response("handler not exist!")
            return

        handler(self._build_ctx())



# -*- coding: UTF-8 -*-
import os
import json
import shutil
from .dataset import IcvDataSet
from .core.meta import SampleMeta, AnnoMeta
from .core.sample import Sample, Anno
from .core.bbox import BBox
from .core.polys import Polygon
from pycocotools.coco import COCO
from ..utils import fcopy, is_dir
from ..image import imwrite
import numpy as np
from tqdm import tqdm

class Coco(IcvDataSet):
    def __init__(self, image_dir, anno_file, keep_no_anno_image=True, one_index=True, transform=None,
                 target_transform=None):
        assert os.path.isdir(image_dir), "param image_dir is not a dir!"
        assert os.path.exists(anno_file), "param anno_file is not exist!"

        self.image_dir = image_dir
        self.anno_file = anno_file
        self.split = os.path.basename(anno_file).rsplit(".", 1)[0]

        self.keep_no_anno_image = keep_no_anno_image
        self.coco = COCO(self.anno_file)

        # self.ids = list(self.coco.imgs.keys())

        self.transform = transform
        self.target_transform = target_transform

        self.categories = self.get_categories()
        self.ids = self.coco.getImgIds()

        self.sample_db = {}
        self.color_map = {}
        super(Coco, self).__init__(self.ids, self.categories, self.keep_no_anno_image, one_index, False)

    def concat(self, coco, out_dir=None, reset=False, new_split=None):
        assert isinstance(coco, Coco)
        split = self.split if new_split is None else new_split
        dist_anno_path, dist_image_path = Coco.reset_dir(out_dir, split, reset=reset)
        dist_anno_file = os.path.join(dist_anno_path, "%s.json" % split)

        # TODO: concat logic
        anno_dict = json.load(open(self.anno_file, "r"))
        # copy image file
        fcopy([os.path.join(self.image_dir, self.coco.loadImgs(id)[0]['file_name']) for id in self.ids],
              dist_image_path)

        max_img_id = np.max(np.uint(self.ids))
        max_anno_id = np.max(list(self.coco.anns.keys()))
        max_cat_id = np.max(list(self.cat2id.values()))

        new_anno_id = max_anno_id + 1
        new_cat_id = max_cat_id + 1

        cat2id = dict(self.cat2id)
        image_id_map = {}
        for i, id in enumerate(coco.ids):
            sample = coco.get_sample(id)
            file_name = sample.meta.file_name
            dist_image_file = os.path.join(dist_image_path, file_name)
            new_filename = fcopy(os.path.join(coco.image_dir, file_name), dist_image_file)

            image_info = sample.meta.dict()
            image_info["id"] = max_img_id + i + 1
            image_info["file_name"] = new_filename

            anno_dict["images"].append(image_info)

            image_id_map[id] = image_info["image_id"]

            for anno in sample.annos:
                anno_meta = anno.meta.dict()
                anno_meta["id"] = new_anno_id
                new_anno_id += 1
                anno_meta["image_id"] = image_info["id"]
                if anno.label in cat2id:
                    anno_meta["category_id"] = cat2id[anno.label]
                else:
                    cat2id[anno.label] = new_cat_id
                    new_cat_id += 1
                    anno_meta["category_id"] = cat2id[anno.label]

                anno_dict["annotations"].append(anno_meta)

        anno_dict["categories"] = [{"supercategory": cat, "id": cat2id[cat], "name": cat} for cat in cat2id]

        json.dump(anno_dict, open(dist_anno_file, "w+"))

        return Coco(
            image_dir=dist_image_path,
            anno_file=dist_anno_file,
            keep_no_anno_image=self.keep_no_anno_image,
            one_index=self.one_index,
            transform=self.transform,
            target_transform=self.target_transform
        )

    @staticmethod
    def reset_dir(dist_dir, split, reset=False):
        if not reset:
            assert is_dir(dist_dir)

        if os.path.exists(dist_dir):
            shutil.rmtree(dist_dir)

        dist_anno_path = os.path.join(dist_dir, "annotations")
        dist_image_path = os.path.join(dist_dir, "images", split)

        for _path in [dist_anno_path, dist_image_path]:
            if reset or not is_dir(_path):
                os.makedirs(_path)

        return dist_anno_path, dist_image_path

    def get_categories(self):
        categories = []
        self.id2cat = {}
        for catid in self.coco.getCatIds():
            categories.append(self.coco.cats[catid]["name"])
            self.id2cat[catid] = self.coco.cats[catid]["name"]
        self.cat2id = {self.id2cat[id]: id for id in self.id2cat}
        return categories

    def get_sample(self, id):
        if id in self.sample_db:
            return self.sample_db[id]

        path = self.coco.loadImgs(id)[0]['file_name']
        image_file = os.path.join(self.image_dir, path)

        annos = []
        for ann in self.coco.imgToAnns[id]:
            xmin, ymin, width, height = ann["bbox"]
            cat = self.id2cat[ann["category_id"]]
            annos.append(Anno(
                bbox=BBox(xmin=xmin, ymin=ymin, xmax=xmin + width, ymax=ymin + height, label=cat),
                polys=Polygon(ann["segmentation"][0], label=cat) if "segmentation" in ann else None,
                label=cat,
                color=self.color_map[cat],
                meta=AnnoMeta({k: ann[k] for k in ann if k not in ["bbox", "category_id", "segmentation"]})
            ))

        img_info = self.coco.imgs[id]
        sample = Sample(
            name=path.rsplit(".", 1)[0],
            image=image_file,
            annos=annos,
            meta=SampleMeta(img_info)
        )
        sample.id = id
        self.sample_db[id] = sample
        return sample

    def write_sample(self, dist_dir):
        dist_anno_path, dist_image_path = Coco.reset_dir(dist_dir, self.split)
        annotation = {
            "images": [],
            "type": "instances",
            "annotations": [],
            "categories": [],
        }

        anno_id = 1
        cats = {}
        if os.path.isdir(dist_dir):
            for id in self.ids:
                sample = self.get_sample(id)
                imwrite(sample.image, os.path.join(dist_image_path, sample.meta.file_name))
                img_height, img_width = sample.image.shape[:2]
                annotation["images"].append(
                    {
                        "id": id,
                        "file_name": sample.meta.file_name,
                        "width": img_width,
                        "height": img_height
                    }
                )

                for anno in sample.annos:
                    anno_dict = anno.meta.dict()
                    anno_dict["bbox"] = [anno.bbox.xmin, anno.bbox.ymin, anno.bbox.width, anno.bbox.height]
                    if anno.seg_mode_polys:
                        anno_dict["segmentation"] = [anno.polys.exterior.flatten().tolist()]
                    elif anno.seg_mode_mask:
                        anno_dict["segmentation"] = [anno.mask.to_ploygons().exterior.flatten().tolist()]

                    anno_dict["category_id"] = self.cat2id[anno.label]
                    anno_dict["image_id"] = id
                    anno_dict["id"] = anno_id
                    anno_id += 1

                    annotation["annotations"].append(anno_dict)

                    if anno.label not in cats:
                        cats[anno.label] = {"supercategory": "", "id": anno["category_id"], "name": anno.label}

        annotation["categories"] = cats.values()
        json.dump(annotation, open(os.path.join(dist_anno_path, "%s.json" % self.split), "w"))

    # def _write_sample(self, anno_samples, dist_path):
    #     annotation = {
    #         "images": [],
    #         "annotations": [],
    #         "categories": [],
    #     }
    #
    #     anno_id = 0
    #     cats = {}
    #     for id, sample in enumerate(anno_samples):
    #         if sample.has_field("file_name"):
    #             continue
    #         img_height, img_width = sample.image.shape[:2]
    #         annotation["images"].append(
    #             {
    #                 "id": id,
    #                 "file_name": sample.get_field("file_name"),
    #                 "width": img_width,
    #                 "height": img_height
    #             }
    #         )
    #
    #         for bbox in sample.bbox_list:
    #             if bbox.has_field("label"):
    #                 continue
    #             anno = {
    #                 "bbox": [bbox.xmin, bbox.ymin, bbox.width, bbox.height]
    #             }
    #
    #             if bbox.has_field("segmentation"):
    #                 anno["segmentation"] = bbox.get_field("segmentation")
    #
    #             if bbox.has_field("area"):
    #                 anno["area"] = bbox.get_field("area")
    #
    #             if bbox.has_field("iscrowd"):
    #                 anno["iscrowd"] = bbox.get_field("iscrowd")
    #
    #             anno["category_id"] = self.cat2id[bbox.lable]
    #             anno["image_id"] = id
    #             anno["id"] = anno_id
    #             anno_id += 1
    #
    #             if bbox.lable not in cats:
    #                 cats[bbox.lable] = {"supercategory": "", "id": anno["category_id"], "name": bbox.lable}
    #
    #     annotation["categories"] = cats.values()
    #     json.dump(annotation, open(dist_path, "w"))

    def showAnns(self, id, with_bbox=True, with_seg=True, is_show=True, save_path=None):
        """
        Display the specified annotations.
        :param anns (array of object): annotations to display
        :return: None
        """
        sample = self.get_sample(id)
        return sample.vis(with_bbox=with_bbox, with_seg=with_seg, is_show=is_show, save_path=save_path)

    def vis(self, id=None, with_bbox=True, with_seg=True, is_show=False, save_dir=None, reset_dir=False):
        if save_dir is not None:
            if not os.path.exists(save_dir):
                os.makedirs(save_dir)
            elif reset_dir:
                shutil.rmtree(save_dir)
                os.makedirs(save_dir)

        if id is not None:
            sample = self.get_sample(id)
            save_path = None if save_dir is None else os.path.join(save_dir, "%s.jpg" % sample.name)
            return self.showAnns(id, with_bbox=with_bbox, with_seg=with_seg, is_show=is_show, save_path=save_path)

        image_vis = []
        for id in tqdm(self.ids):
            sample = self.get_sample(id)
            save_path = None if save_dir is None else os.path.join(save_dir, "%s.jpg" % sample.name)
            image = self.showAnns(id, with_bbox=with_bbox, with_seg=with_seg, is_show=False, save_path=save_path)
            image_vis.append(image)
        return image_vis



# -*- coding: UTF-8 -*-
from ..vis.color import STANDARD_COLORS
from abc import ABCMeta, abstractmethod
from collections import OrderedDict
from terminaltables import AsciiTable
import random
import copy


class IcvDataSet(object):
    __metaclass__ = ABCMeta

    def __init__(self, ids=None, categories=None, keep_no_anno_image=True, one_index=False, build_cat=True):
        self.ids = ids if ids else []
        self.categories = categories if categories else []
        self.keep_no_anno_image = keep_no_anno_image
        self.one_index = one_index
        self.num_classes = len(self.categories)

        if build_cat:
            self._build()

        if not hasattr(self, "color_map") or self.color_map is None or len(self.color_map) == 0:
            self.set_colormap()

    def _build(self):
        if self.one_index:
            self.id2cat = {i + 1: cat for i, cat in enumerate(self.categories)}
            self.cat2id = {cat: i + 1 for i, cat in enumerate(self.categories)}
        else:
            self.id2cat = {i: cat for i, cat in enumerate(self.categories)}
            self.cat2id = {cat: i for i, cat in enumerate(self.categories)}

        self.id2index = {id: i for i, id in enumerate(self.ids)}
        self.index2id = {i: id for i, id in enumerate(self.ids)}

    def get_img_info(self, index):
        img_id = self.ids[index]
        return img_id

    def set_colormap(self):
        self.color_map = {}
        _colors = STANDARD_COLORS
        # random.shuffle(_colors)
        for i, cat in enumerate(self.categories):
            self.color_map[cat] = _colors[i % len(_colors)]

    @property
    def length(self):
        return len(self)

    def __len__(self):
        return len(self.ids)

    def __getitem__(self, item):
        return self.get_sample(self.index2id[item])

    def get_categories(self):
        return self.categories

    def get_samples(self, ids=None):
        self.samples = []
        ids = ids if ids is not None else self.ids
        for id in ids:
            sample = self.get_sample(id)
            if sample.count == 0 and not self.keep_no_anno_image:
                continue
            self.samples.append(sample)

        return self.samples

    def get_sample(self, id):
        pass

    def get_groundtruth(self, id):
        return self.get_sample(id)

    @abstractmethod
    def vis(self, save_dir=None):
        pass

    @abstractmethod
    def sub(self, count=0, ratio=0, shuffle=True, output_dir=None):
        if count <= 0:
            count = round(self.length * ratio)

        count = min(self.length, count)
        if shuffle:
            ids = random.sample(self.ids, count)
        else:
            ids = self.ids[:count]

        dataset = copy.deepcopy(self)
        dataset.ids = ids
        dataset.categories = dataset.get_categories()

        if output_dir is not None:
            dataset.save(output_dir)

        return dataset

    @abstractmethod
    def save(self, output_dir, split=None):
        pass

    @abstractmethod
    def concat(self, dataset, out_dir, reset=False, new_split=None):
        pass

    @abstractmethod
    def statistic(self, print_log=False):
        samples = self.get_samples()

        sta = dict(overview=dict(), detail=dict())
        sta["overview"] = {
            "image_num": self.length,
            "class_num": len(self.categories),
            "bbox_num": sum([sample.count for sample in samples]),
            "min_width": min([sample.width for sample in samples]),
            "max_width": max([sample.width for sample in samples]),
            "min_height": min([sample.height for sample in samples]),
            "max_height": max([sample.height for sample in samples]),
            "same_size": len(set([(sample.height, sample.width) for sample in samples])) == 1,
            "ratio_distribution": OrderedDict(),
        }

        sta["detail"] = {
            cat: {
                "num": 0,
                "ratio_distribution": OrderedDict(),
            }
            for cat in self.categories
        }

        for sample in samples:
            s = sample.statistics()
            for label in s["cats"]:
                sta["detail"][label]["num"] = s["cats"][label]

            for label in s["ratios"]:
                for r in s["ratios"][label]:
                    ratio = str(r)
                    if ratio not in sta["overview"]["ratio_distribution"]:
                        sta["overview"]["ratio_distribution"][ratio] = 0
                    sta["overview"]["ratio_distribution"][ratio] += s["ratios"][label][r]

                    if ratio not in sta["detail"][label]["ratio_distribution"]:
                        sta["detail"][label]["ratio_distribution"][ratio] = 0
                    sta["detail"][label]["ratio_distribution"][ratio] += s["ratios"][label][r]

        if print_log:
            overview_header = ["image_num", "class_num", "bbox_num", "same_image_size",
                               "min_image_width", "max_image_width", "min_image_height", "max_image_height"]
            overview_table_data = [
                overview_header,
                [
                    sta["overview"]["image_num"], sta["overview"]["class_num"], sta["overview"]["bbox_num"],
                    sta["overview"]["same_size"], sta["overview"]["min_width"], sta["overview"]["max_width"],
                    sta["overview"]["min_height"], sta["overview"]["max_height"],
                ]
            ]
            overview_table = AsciiTable(overview_table_data)
            overview_table.inner_footing_row_border = True

            ratios = sorted(list(sta["overview"]["ratio_distribution"].keys()))

            ratio_distri_header = ["bbox ratio (h/w)"] + ratios
            ratio_distri_table_data = [
                ratio_distri_header,
                ["-"] + [sta["overview"]["ratio_distribution"][r] for r in ratios]
            ]

            ratio_distri_table = AsciiTable(ratio_distri_table_data)
            ratio_distri_table.inner_footing_row_border = True

            cats_detail_header = ["bbox ratio (h/w) \ class"] + self.categories
            cats_detail_table_data = [cats_detail_header]
            cats_detail_table_data.append(["num"] + [sta["detail"][cat]["num"] for cat in self.categories])
            for r in ratios:
                cats_detail_table_data.append(
                    [r] + [sta["detail"][cat]["ratio_distribution"].get(r, 0) for cat in self.categories])

            cats_detail_table = AsciiTable(cats_detail_table_data)
            cats_detail_table.inner_footing_row_border = True

            print("overview:")
            print(overview_table.table)

            print("\nratio distribution:")
            print(ratio_distri_table.table)

            print("\nclass detail:")
            print(cats_detail_table.table)

        return sta



# -*- coding: UTF-8 -*-
# from lxml import etree
from xml.etree import ElementTree as etree
import os
import shutil

def load_voc_anno(anno_path,filter_empty=True):
    """Recursively parses XML contents to python dict.

    We assume that `object` tags are the only ones that can appear
    multiple times at the same level of a tree.

    Args:
      xml: xml tree obtained by parsing XML file contents using lxml.etree

    Returns:
      Python dictionary holding XML contents.
    """
    def _load_anno(xml):
        if not xml:
            return {xml.tag: xml.text}
        result = {}
        for child in xml:
            child_result = _load_anno(child)
            if child.tag != 'object':
                result[child.tag] = child_result[child.tag]
            else:
                if child.tag not in result:
                    result[child.tag] = []
                result[child.tag].append(child_result[child.tag])
        return {xml.tag: result}

    with open(anno_path, 'r') as fid:
        xml_str = fid.read()
    if "<bndbox>" not in xml_str and filter_empty:
        return {}
    xml = etree.fromstring(xml_str)
    return _load_anno(xml)

def make_empty_voc_anno(**kwargs):
    empty_anno = {
        "folder":"",
        "filename":"",
        "source":{
            "database":"The VOC2012 Database",
            "annotation":"PASCAL VOC2007",
            "image":"flickr",
        },
        "size":{
            "width":"0",
            "height":"0",
            "depth":"0"
        },
        "segmented":"0",
        "object":{},
    }

    if "folder" in kwargs:
        empty_anno["folder"] = kwargs["folder"]

    if "filename" in kwargs:
        empty_anno["filename"] = kwargs["filename"]

    if "width" in kwargs:
        empty_anno["size"]["width"] = kwargs["width"]

    if "height" in kwargs:
        empty_anno["size"]["height"] = kwargs["height"]

    if "depth" in kwargs:
        empty_anno["size"]["depth"] = kwargs["depth"]

    return empty_anno

def reset_voc_dir(dist_dir,with_segment=True):
    if not os.path.exists(dist_dir):
        os.makedirs(dist_dir)
    shutil.rmtree(dist_dir)

    dist_image_path = os.path.join(dist_dir, "JPEGImages")
    dist_anno_path = os.path.join(dist_dir, "Annotations")
    dist_imageset_path = os.path.join(dist_dir, "ImageSets", "Main")

    dist_segmentation_imageset_path = os.path.join(dist_dir, "Segmentation", "Main")
    dist_segmentation_class_image_path = os.path.join(dist_dir, "SegmentationClass")
    dist_segmentation_object_image_path = os.path.join(dist_dir, "SegmentationObject")

    os.makedirs(dist_image_path)
    os.makedirs(dist_anno_path)
    os.makedirs(dist_imageset_path)

    if with_segment:
        os.makedirs(dist_segmentation_imageset_path)
        os.makedirs(dist_segmentation_class_image_path)
        os.makedirs(dist_segmentation_object_image_path)

        return dist_image_path, dist_anno_path, dist_imageset_path, \
               dist_segmentation_imageset_path, dist_segmentation_class_image_path, dist_segmentation_object_image_path

    return dist_image_path, dist_anno_path, dist_imageset_path

def reset_coco_dir(dist_dir):
    if not os.path.exists(dist_dir):
        os.makedirs(dist_dir)
    shutil.rmtree(dist_dir)

    anno_dir = os.path.join(dist_dir,"annotations")
    train_image_dir = os.path.join(dist_dir,"train")
    test_image_dir = os.path.join(dist_dir,"test")
    val_image_dir = os.path.join(dist_dir,"val")

    os.makedirs(anno_dir)
    os.makedirs(train_image_dir)
    os.makedirs(test_image_dir)
    os.makedirs(val_image_dir)

    return anno_dir,train_image_dir,test_image_dir,val_image_dir



# -*- coding: UTF-8 -*-
from icv.utils import is_valid_url
from ..methods import HttpMethod
from .request import IcvHttpRequest
import json as pjson

class IcvHttpClient(object):
    def __init__(self,url,data=None,params=None,headers=None,timeout=3000):
        if not is_valid_url(url):
            raise Exception("request url is invalid")

        self.method = "GET"
        self.url = url
        self.data = data
        self.params = params if params else {}
        self.headers = headers if headers else {}
        self.timeout = timeout

    def set_method(self,method):
        method = str(method).upper()
        if method not in HttpMethod:
            raise Exception("request method invalid")
        self.method = method

    def set_headers(self,headers):
        self.headers = headers if headers else {}

    def header(self,key,value):
        self.headers[key] = value

    def set_data(self,data):
        self.data = data

    def set_params(self,params):
        self.params = params if params else {}

    def set_timeout(self,timeout):
        self.timeout = timeout

    def do_request(self):
        req = IcvHttpRequest(self.method,self.url,data=self.data,params=self.params,headers=self.headers,timeout=self.timeout)
        return req.send()

    def post(self,json=False,form=False):
        if json:
            if isinstance(self.data,dict):
                self.data = pjson.dumps(self.data).encode("utf-8")
            self.header("Content-Type","application/json")
        elif form:
            if isinstance(self.data,dict):
                self.data = "&".join(["{}={}".format(k,self.data[k]) for k in self.data])
            self.header("Content-Type","application/x-www-form-urlencoded")

        self.method = HttpMethod.POST
        return self.do_request()

    def get(self):
        self.method = HttpMethod.GET

        return self.do_request()

    def put(self,json=False,form=False):
        if json:
            if isinstance(self.data,dict):
                self.data = pjson.dumps(self.data).encode("utf-8")
            self.header("Content-Type","application/json")
        elif form:
            if isinstance(self.data,dict):
                self.data = "&".join(["{}={}".format(k,self.data[k]) for k in self.data])
            self.header("Content-Type","application/x-www-form-urlencoded")

        self.method = HttpMethod.PUT
        return self.do_request()

    def delete(self):
        self.method = HttpMethod.DELETE
        return self.do_request()

    def options(self):
        self.method = HttpMethod.OPTIONS
        return self.do_request()

    def __repr__(self):
        return "<IcvHttpClient> [{}] {}".format(self.method,self.url)





# -*- coding: UTF-8 -*-
from abc import ABCMeta,abstractclassmethod
from icv.utils import EasyDict

class Meta(object):
    __metaclass__ = ABCMeta

class SampleMeta(EasyDict):
    pass

class AnnoMeta(EasyDict):
    pass






from .coco import Coco
from .voc import Voc
from .dataset import IcvDataSet
from .labelme import LabelMe
from .classify import Classify

__all__ = ['Coco', 'Voc', 'IcvDataSet', 'LabelMe', 'Classify']



# -*- coding: UTF-8 -*-
try:
    import mmdet
except ModuleNotFoundError as e:
    raise Exception("You should install mmdetection first " \
                    "(https://github.com/open-mmlab/mmdetection/blob/master/INSTALL.md) . " \
                    "Import mmdet Error!")

MMDET_VERSION = int(mmdet.__version__.split("+")[0].replace(".",""))

import os
import warnings
import numpy as np
from ..data.core import BBox
from ..utils import ckpt_load, Config, Timer, is_file, concat_list
from ..image import imread, imwrite, imshow, imresize
from .detector import Detector
from .result import DetectionResult
import pycocotools.mask as maskUtils
from mmdet.core import get_classes
from mmdet.models import build_detector
from mmdet.apis import inference_detector

class MmdetDetector(Detector):
    def __init__(self, model_path, config_file, categories, iou_thr=0.5, score_thr=0.5, device=None):
        super(MmdetDetector, self).__init__(categories=categories, iou_thr=iou_thr, score_thr=score_thr, device=device)

        self.model_path = model_path
        self.config_file = config_file
        self._build_detector()

    def _build_detector(self):
        self.model = self._init_detector(self.config_file, self.model_path)

    def _init_detector(self, config, checkpoint=None, device='cuda:0'):
        """Initialize a detector from config file.

        Args:
            config (str or :obj:`mmcv.Config`): Config file path or the config
                object.
            checkpoint (str, optional): Checkpoint path. If left as None, the model
                will not load any weights.

        Returns:
            nn.Module: The constructed detector.
        """
        if isinstance(config, str):
            self.cfg = Config.fromfile(config)
        elif not isinstance(config, Config):
            raise TypeError('config must be a filename or Config object, '
                            'but got {}'.format(type(config)))
        self.cfg.model.pretrained = None
        model = build_detector(self.cfg.model, test_cfg=self.cfg.test_cfg)
        if checkpoint is not None:
            checkpoint = ckpt_load(model, checkpoint)
            if 'CLASSES' in checkpoint['meta']:
                model.CLASSES = checkpoint['meta']['CLASSES']
            else:
                warnings.warn('Class names are not saved in the checkpoint\'s '
                              'meta data, use COCO classes by default.')
                model.CLASSES = get_classes('coco')
        model.cfg = self.cfg  # save the config in the model for convenience
        model.to(device)
        model.eval()
        return model

    def _build_result(self, inference_result, inference_time=0, score_thr=-1):
        if isinstance(inference_result, tuple):
            bbox_result, segm_result = inference_result
        else:
            bbox_result, segm_result = inference_result, None

        detection_masks = []
        if segm_result is not None:
            segms = concat_list(segm_result)
            for segm in segms:
                mask = maskUtils.decode(segm)
                detection_masks.append(mask)

        detection_masks = np.array(detection_masks)

        preds = [
            np.full(bbox.shape[0], i, dtype=np.int32)
            for i, bbox in enumerate(bbox_result)
        ]

        detection_classes = np.concatenate(preds)
        bbox_result = np.vstack(bbox_result)

        detection_bboxes = []
        detection_scores = []

        for ix in range(bbox_result.shape[0]):
            detection_bboxes.append(
                BBox(bbox_result[ix, 0], bbox_result[ix, 1], bbox_result[ix, 2], bbox_result[ix, 3]))
            detection_scores.append(bbox_result[ix, 4])

        score_thr = score_thr if score_thr >= 0 else self.score_thr
        det_result = DetectionResult(
            det_bboxes=detection_bboxes,
            det_classes=detection_classes + 1,
            det_scores=detection_scores,
            det_masks=detection_masks,  # TODO: process segm_result
            det_time=inference_time,
            categories=self.categories,
            score_thr=score_thr
        )

        return det_result

    def inference(self, image, is_show=False, save_path=None, score_thr=-1):
        image_np = imread(image)
        timer = Timer()
        result = inference_detector(self.model, image_np)
        inference_time = timer.since_start()

        score_thr = score_thr if score_thr >= 0 else self.score_thr
        det_result = self._build_result(result, inference_time, score_thr)
        det_result.vis(image_np)

        if is_show:
            imshow(det_result.det_image)

        if save_path is not None:
            imwrite(det_result.det_image, save_path)

        return det_result

    def inference_batch(self, image_batch, save_dir=None, resize=None, score_thr=-1):
        image_np_list = [imread(img) for img in image_batch]
        if resize:
            image_np_list = [imresize(img_np, resize) for img_np in image_np_list]

        timer = Timer()
        results = inference_detector(self.model, image_np_list)

        inference_time = timer.since_start()
        score_thr = score_thr if score_thr >= 0 else self.score_thr

        det_result_list = []
        for ix, result in enumerate(results):
            det_result = self._build_result(result, inference_time, score_thr)
            det_result.vis(image_np_list[ix])

            if save_dir is not None:
                save_path = os.path.join(save_dir, os.path.basename(image_batch[ix])) if is_file(
                    image_batch[0]) else os.path.join(save_dir, str(ix) + ".jpg")
                imwrite(det_result.det_image, save_path)

            det_result_list.append(det_result)

        return det_result_list



from .io import imread,imread_topil,imread_tob64,imwrite,np_img_to_pil,pil_img_to_np
from .vis import imshow,immerge,imshow_bboxes,imdraw_bbox,imdraw_mask,imdraw_polygons,imdraw_polygons_with_bbox
from .transforms import imresize, imresize_like, imrescale

__all__ = [
    'imread','imread_topil','imread_tob64','imwrite','np_img_to_pil','pil_img_to_np',
    'imshow','immerge','imshow_bboxes','imdraw_bbox','imdraw_mask','imdraw_polygons','imdraw_polygons_with_bbox',
    'imresize', 'imresize_like', 'imrescale'
]



# -*- coding: UTF-8 -*-

STANDARD_COLORS = [
    'AliceBlue', 'Chartreuse', 'Aqua', 'Aquamarine', 'Azure', 'Beige', 'Bisque',
    'BlanchedAlmond', 'BlueViolet', 'BurlyWood', 'CadetBlue', 'AntiqueWhite',
    'Chocolate', 'Coral', 'CornflowerBlue', 'Cornsilk', 'Crimson', 'Cyan',
    'DarkCyan', 'DarkGoldenRod', 'DarkGrey', 'DarkKhaki', 'DarkOrange',
    'DarkOrchid', 'DarkSalmon', 'DarkSeaGreen', 'DarkTurquoise', 'DarkViolet',
    'DeepPink', 'DeepSkyBlue', 'DodgerBlue', 'FireBrick', 'FloralWhite',
    'ForestGreen', 'Fuchsia', 'Gainsboro', 'GhostWhite', 'Gold', 'GoldenRod',
    'Salmon', 'Tan', 'HoneyDew', 'HotPink', 'IndianRed', 'Ivory', 'Khaki',
    'Lavender', 'LavenderBlush', 'LawnGreen', 'LemonChiffon', 'LightBlue',
    'LightCoral', 'LightCyan', 'LightGoldenRodYellow', 'LightGray', 'LightGrey',
    'LightGreen', 'LightPink', 'LightSalmon', 'LightSeaGreen', 'LightSkyBlue',
    'LightSlateGray', 'LightSlateGrey', 'LightSteelBlue', 'LightYellow', 'Lime',
    'LimeGreen', 'Linen', 'Magenta', 'MediumAquaMarine', 'MediumOrchid',
    'MediumPurple', 'MediumSeaGreen', 'MediumSlateBlue', 'MediumSpringGreen',
    'MediumTurquoise', 'MediumVioletRed', 'MintCream', 'MistyRose', 'Moccasin',
    'NavajoWhite', 'OldLace', 'Olive', 'OliveDrab', 'Orange', 'OrangeRed',
    'Orchid', 'PaleGoldenRod', 'PaleGreen', 'PaleTurquoise', 'PaleVioletRed',
    'PapayaWhip', 'PeachPuff', 'Peru', 'Pink', 'Plum', 'PowderBlue', 'Purple',
    'Red', 'RosyBrown', 'RoyalBlue', 'SaddleBrown', 'Green', 'SandyBrown',
    'SeaGreen', 'SeaShell', 'Sienna', 'Silver', 'SkyBlue', 'SlateBlue',
    'SlateGray', 'SlateGrey', 'Snow', 'SpringGreen', 'SteelBlue', 'GreenYellow',
    'Teal', 'Thistle', 'Tomato', 'Turquoise', 'Violet', 'Wheat', 'White',
    'WhiteSmoke', 'Yellow', 'YellowGreen'
]

MASK_COLORS = [
    (0,0,139),
    (67,205,128),
    (0,206,209),
    (255,106,106),
    (192,255,62),
    (130,130,130),
    (139,87,66),
    (25,25,112)
]


# -*- coding: UTF-8 -*-
import os
from PIL import Image, ImageDraw, ImageFont, ImageColor
from .io import imread, imwrite, pil_img_to_np
from ..utils import is_seq, is_empty, is_np_array, is_str
from ..vis.color import STANDARD_COLORS, MASK_COLORS
import random
import cv2
import numpy as np
import matplotlib.pyplot as plt


def imshow(img):
    """Show an image.
    Args:
        img (str or ndarray): The image to be displayed.
    """
    plt.axis('off')
    plt.imshow(imread(img))
    plt.show()


def immerge(img_list, origin="x", resize=False):
    assert origin in ["x", "y"], "param origin should be 'x' or 'y'"
    assert is_seq(img_list) and not is_empty(img_list), "param img_list should be a sequence"
    if len(img_list) == 1:
        return imread(img_list[0])

    imgnp_list = []
    shape = ()
    for img in img_list:
        imgnp = imread(img)
        if not is_empty(shape):
            if resize:
                imgnp = cv2.resize(imgnp, shape)
            if (origin == "x" and imgnp.shape[0] != shape[0]) or (origin == "y" and imgnp.shape[1] != shape[1]):
                raise Exception("image shape must match exactly or use resize=True.")
        shape = (imgnp.shape[0], imgnp.shape[1])
        imgnp_list.append(imgnp)

    merged_img_np = np.concatenate(imgnp_list, axis=1 if origin == "x" else 0)
    return merged_img_np


def imshow_bboxes(
        img,
        bboxes,
        labels=None,
        scores=None,
        classes=None,
        score_thresh=0,
        masks=None,
        color="red",
        thickness=1,
        use_normalized_coordinates=False,
        is_show=False,
        save_path=None):
    assert classes is None or is_seq(classes) or is_np_array(classes)
    assert labels is None or is_seq(labels) or is_np_array(labels)
    assert scores is None or is_seq(scores) or is_np_array(scores)
    assert masks is None or is_seq(masks) or is_np_array(masks)

    from icv.data.core import BBoxList, BBox

    if isinstance(bboxes, BBoxList):
        bboxes = np.array(bboxes.tolist())
    elif isinstance(bboxes, list) and len(bboxes) > 0 and isinstance(bboxes[0], BBox):
        bboxes = np.array([bbox.bbox for bbox in bboxes])
    else:
        bboxes = np.array(bboxes)

    image = imread(img)
    image_pil = Image.fromarray(np.uint8(image)).convert('RGB')

    if bboxes.shape[0] == 0:
        if is_show:
            imshow(image)

        if save_path:
            imwrite(image, save_path)

        return image

    bboxes = bboxes[np.where(np.array(scores) >= score_thresh)] if scores is not None else bboxes

    if labels is not None:
        if not is_np_array(labels):
            labels = np.array(labels)
        labels = labels[np.where(scores >= score_thresh)] if scores is not None else labels
        assert labels.shape[0] == bboxes.shape[0], "param labels's length is not equals to bboxes!"

    if masks is not None:
        if not is_np_array(masks):
            masks = np.array(masks)
        masks = masks[np.where(scores >= score_thresh)] if scores is not None else masks
        assert masks.shape[0] == bboxes.shape[0], "param masks's length is not equals to bboxes!"

    if scores is not None:
        if not is_np_array(scores):
            scores = np.array(scores)
        scores = scores[np.where(scores >= score_thresh)]

        assert scores.shape[0] == bboxes.shape[0], "param scores's length is not equals to bboxes!"

    colorMap = {}
    default_color = color if color else "DarkOrange"
    if classes is not None:
        if is_np_array(classes):
            classes = list(classes)
        for cat in classes:
            colorMap[cat] = STANDARD_COLORS[classes.index(cat) % len(STANDARD_COLORS)]

    for ix, bbox in enumerate(bboxes):
        if isinstance(bbox, BBox):
            label = labels[ix] if labels is not None else bbox.lable
            color = colorMap[label] if label in colorMap else default_color
            label = label if label is not None else ""
            label = label + ": " + str(round(scores[ix], 3)) if scores is not None else label
        else:
            label = labels[ix] if labels is not None else ""
            color = colorMap[label] if label in colorMap else default_color
            label = label + ": " + str(round(scores[ix], 3)) if scores is not None else label

        if masks is not None:
            image_pil = imdraw_mask(image_pil, masks[ix], color=color)

        if isinstance(bbox, BBox):
            image_pil = imdraw_bbox(image_pil, bbox.xmin, bbox.ymin, bbox.xmax, bbox.ymax, color, thickness, label,
                                    use_normalized_coordinates)
        else:
            image_pil = imdraw_bbox(image_pil, bbox[0], bbox[1], bbox[2], bbox[3], color, thickness, label,
                                    use_normalized_coordinates)

    np.copyto(image, np.array(image_pil))

    if is_show:
        imshow(image)

    if save_path:
        imwrite(image, save_path)

    return image

def imdraw_bbox(image, xmin, ymin, xmax, ymax, color="red", thickness=1, display_str="",
                use_normalized_coordinates=False):
    assert xmin <= xmax, "xmin shouldn't be langer than xmax!"
    assert ymin <= ymax, "ymin shouldn't be langer than ymax!"

    if isinstance(image, np.ndarray):
        image = Image.fromarray(image).convert('RGB')

    if xmin == xmax or ymin == ymax:
        return image

    draw = ImageDraw.Draw(image)
    im_height, im_width = image.size
    if use_normalized_coordinates:
        (left, right, top, bottom) = (xmin * im_width, xmax * im_width,
                                      ymin * im_height, ymax * im_height)
    else:
        (left, right, top, bottom) = (xmin, xmax, ymin, ymax)
    draw.line([(left, top), (left, bottom), (right, bottom),
               (right, top), (left, top)], width=thickness, fill=color)

    if display_str == "":
        return image

    rgb = ImageColor.getrgb(color) if is_str(color) else color
    rgba = rgb + (128,)

    try:
        font = ImageFont.truetype('arial.ttf', 24)
    except IOError:
        font = ImageFont.load_default()

    display_str_heights = font.getsize(display_str)[1]
    # Each display_str has a top and bottom margin of 0.05x.
    total_display_str_height = (1 + 2 * 0.05) * display_str_heights

    if top > total_display_str_height:
        text_bottom = top
    else:
        text_bottom = bottom + total_display_str_height

    text_width, text_height = font.getsize(display_str)
    margin = np.ceil(0.05 * text_height)
    if text_width == 0:
        text_width = len(display_str) * 7
    draw.rectangle(
        [(left, text_bottom - text_height - 2 * margin), (left + text_width,
                                                          text_bottom)],
        fill=rgba)
    draw.text(
        (left + margin, text_bottom - text_height - margin),
        display_str,
        fill='black',
        font=font)

    return pil_img_to_np(image)


def imdraw_mask(image, mask, color='red', alpha=0.8):
    """Draws mask on an image.

    Args:
      image: uint8 numpy array with shape (img_height, img_height, 3)
      mask: a uint8 numpy array of shape (img_height, img_height) with
        values between either 0 or 1.
      color: color to draw the keypoints with. Default is red.
      alpha: transparency value between 0 and 1. (default: 0.4)

    Raises:
      ValueError: On incorrect data type for image or masks.
    """
    if mask.dtype != np.uint8:
        raise ValueError('`mask` not of type np.uint8')
    if np.any(np.logical_and(mask != 1, mask != 0)):
        raise ValueError('`mask` elements should be in [0, 1]')

    image = imread(image)

    if image.shape[:2] != mask.shape:
        raise ValueError('The image has spatial dimensions %s but the mask has '
                         'dimensions %s' % (image.shape[:2], mask.shape))

    color_mask = ImageColor.getrgb(color) if is_str(color) else color
    color_mask = np.array(color_mask, dtype=np.uint8)

    mask_bin = mask.astype(np.bool)
    image[mask_bin] = image[mask_bin] * alpha + color_mask * (1 - alpha)

    return image


def imdraw_polygons(image, polygons, color='red', alpha=0.8):
    image = imread(image)

    polygons = _format_polygons(polygons)

    rgb = ImageColor.getrgb(color) if is_str(color) else color
    alpha = int(alpha * 255) if alpha <= 1 else int(alpha)
    rgba = rgb + (alpha,)
    pil_image = Image.fromarray(image)

    draw = ImageDraw.Draw(pil_image)
    for polygon in polygons:
        points = [tuple(p) for p in polygon]
        draw.polygon(points, fill=rgba)

    np.copyto(image, np.array(pil_image.convert('RGB')))
    return image


def _format_polygons(polygons):
    if is_np_array(polygons):
        polygons = polygons.tolist()
    ps = []
    for polygon in polygons:
        if is_seq(polygon[0]):
            ps.append([tuple(p) for p in polygon])
        else:
            if len(polygon) % 2 == 0:
                ps.append([(polygon[i], polygon[i + 1]) for i in range(len(polygon)) if i % 2 == 0])
    return ps


def _get_bbox_from_points(polygon):
    """
    根据polygon顶点获取bbox
    :param polygon:
    :return:
    """
    x_list = [p[0] for p in polygon]
    y_list = [p[1] for p in polygon]

    xmin = min(x_list)
    ymin = min(y_list)
    xmax = max(x_list)
    ymax = max(y_list)

    return xmin, ymin, xmax, ymax


def imdraw_polygons_with_bbox(
        image,
        polygons,
        name_list=None,
        is_show=False,
        save_path=None,
        alpha=0.5,
        outline=1,
        color_map=None,
        with_bbox=False,
        bbox_color='black',
        text_color='black'
):
    image = imread(image)
    pil_image = Image.fromarray(image)
    draw = ImageDraw.Draw(pil_image, mode="RGBA")

    polygons = _format_polygons(polygons)

    if len(polygons) <= 0:
        if save_path is not None:
            pil_image.save(save_path)

        if is_show:
            imshow(image)
        return image

    assert name_list is None or len(polygons) == len(name_list)
    assert color_map is None or isinstance(color_map, dict)

    color_map_history = {}
    for ix, polygon in enumerate(polygons):
        color = random.choice(MASK_COLORS)

        points = [tuple(p) for p in polygon]
        xmin, ymin, xmax, ymax = _get_bbox_from_points(points)
        (left, right, top, bottom) = (xmin, xmax, ymin, ymax)

        if with_bbox:
            draw.line([(left, top), (left, bottom), (right, bottom),
                       (right, top), (left, top)], width=1, fill=bbox_color)

        try:
            font = ImageFont.truetype('arial.ttf', 24)
        except IOError:
            font = ImageFont.load_default()

        if name_list is not None:
            name = name_list[ix]

            if name in color_map_history:
                color = color_map_history[name]
            elif color_map is not None and name in color_map:
                color = color_map[name]
                color = color if len(color) == 4 else color + (round(255 * alpha),)
                color_map_history[name] = color
            else:
                color = color + (round(255 * alpha),)
                color = color if len(color) == 4 else color + (round(255 * alpha),)
                color_map_history[name] = color

            display_str_heights = font.getsize(name)[1]
            # Each display_str has a top and bottom margin of 0.05x.
            total_display_str_height = (1 + 2 * 0.05) * display_str_heights

            if top > total_display_str_height:
                text_bottom = top
            else:
                text_bottom = bottom + total_display_str_height

            text_width, text_height = font.getsize(name)
            margin = np.ceil(0.05 * text_height)
            draw.rectangle(
                [(left, text_bottom - text_height - 2 * margin), (left + text_width,
                                                                  text_bottom)],
                fill=color)
            draw.text(
                (left + margin, text_bottom - text_height - margin),
                name,
                fill=text_color,
                font=font)
            text_bottom -= text_height - 2 * margin

        draw.polygon(points, fill=color, outline=outline)

    np.copyto(image, np.array(pil_image.convert('RGB')))

    if save_path is not None:
        pil_image.save(save_path)

    if is_show:
        imshow(image)

    return image



# -*- coding: utf-8 -* -
import numpy as np
from io import BytesIO
from PIL import Image
import base64
from .itis import is_np_array

def base64_to_np(b64_code):
    image = Image.open(BytesIO(base64.b64decode(b64_code)))
    img = np.array(image)
    return img

def np_to_base64(image_np):
    assert is_np_array(image_np)
    output_buffer = BytesIO()
    img = Image.fromarray(image_np.astype('uint8')).convert('RGB')
    img.save(output_buffer, format='JPEG')
    binary_data = output_buffer.getvalue()
    base64_data = base64.b64encode(binary_data)
    return base64_data

